use master
if exists(select * from master.dbo.sysobjects where name = 'sp_blocked')
    drop procedure sp_blocked
go
create procedure dbo.sp_blocked
    @spid int
as
create table #Blocked
(spid int)
insert into #Blocked
(spid)
values
(@spid)

while @@ROWCOUNT <> 0
BEGIN
    insert into #Blocked
    (spid)
    select spid 
        from master.dbo.sysprocesses 
        where blocked in (select spid from #Blocked)
        and spid not in (select spid from #Blocked)
END

delete from #Blocked
where spid = @spid

delete from #Blocked
where spid is null

if exists(select * from #Blocked)
BEGIN
    select *
    from master.dbo.sysprocesses
    where spid in (select spid from #blocked)
END
else
BEGIN
    select 'No Processes are being blocked by spid ' + convert(varchar(20), @spid) + '.' as 'System Message'
END

drop table #Blocked
go
print 'sp_blocked created.'

USE master
print 'Creating sp_who3'

if exists (select * from dbo.sysobjects where name = 'sp_who3')
drop procedure [dbo].[sp_who3]
GO
SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE procedure sp_who3
    @spid sysname = null
as
DECLARE @spid_i INT
DECLARE @spid_only bit
SET NOCOUNT ON
if @spid is null
BEGIN
    exec sp_who2
END
ELSE
BEGIN
    set @spid_only = 1
    if lower(cast(@spid as varchar(10))) = 'active'
    BEGIN
        set @spid_only = 0
        exec sp_who2 'active'
    END
    if lower(cast(@spid as varchar(10))) = 'blocked'
    BEGIN
        DECLARE @blocked TABLE
        (spid int,
        blocked int)

        INSERT INTO @blocked
        select spid, blocked
        from sysprocesses
        where blocked <> 0

        insert into @blocked
        select spid, blocked
        from sysprocesses
        where spid in (select blocked from @blocked)

        set @spid_only = 0
        select sysprocesses.spid as 'SPID', 
            sysprocesses.status, 
            sysprocesses.loginame as 'Login',
            sysprocesses.hostname as 'HostName',
            sysprocesses.blocked as 'BlkBy',
            sysdatabases.name as 'DBName',
            sysprocesses.cmd as 'Command',
            sysprocesses.cpu as 'CPUTime',
            sysprocesses.physical_io as 'DiskIO',
            sysprocesses.last_batch as 'LastBatch',
            sysprocesses.program_name as 'ProgramName',
            sysprocesses.spid as 'SPID'
        from sysprocesses
        left outer join sysdatabases on sysprocesses.dbid = sysdatabases.dbid
        where spid in (select spid from @blocked)
    END
    
    if @spid_only = 1
    BEGIN
        set @spid_i = @spid
        exec sp_who @spid_i
        exec sp_who2 @spid_i
        dbcc inputbuffer (@spid_i)
        /*Start Get Output Buffer*/
        if exists (select * from sysobjects where name = 'fn_get_sql')
        BEGIN
            declare @SQL_Handle binary(20)
            select @SQL_Handle = sql_handle
            from master.dbo.sysprocesses
            where spid = @spid_i
            
            if @SQL_Handle is not null
            BEGIN    
                Create Table #SQL
                (SQLText text)
                insert into #SQL
                 SELECT [text]
                    FROM ::fn_get_sql(@SQL_Handle)
                if exists(select * from #SQL)
                    select SQLText 'Full SQL Query' from #SQL
                drop table #SQL
            END
        END
        /*End Get Output Buffer*/
        select * from master.dbo.sysprocesses where spid = @spid_i
        exec sp_blocked @spid_i
        exec sp_lock @spid_i
    END
END
GO
print 'sp_who3 created.'
go
GRANT exec on sp_who3 to public
GO